import { select } from 'd3';
import { color } from './color';
import { appendRandomWord } from './random';
import { words } from './data';

const uploadRandomWord = () => {
	appendRandomWord(words, wordCloud );
  refresh();
}

/* Define the animation we want to do */
const animationFunction = uploadRandomWord;
const animationRateMS = 100;
const animationRateDurationMS = 20000;

/* parameters */
const svgWidth = 940;
const svgHeight = 474;

const transitionDurationMS = 300;


/* location parameters */
const flyInFromX = svgWidth/2;
const flyInFromY = svgHeight/2;
const flyInFromFontSize = 280;
const flyInFromColor = 'none';
const flyInFromOpacity = 0;

const wordSize = 12;
const wordColor = d => color(d.fill);
const wordOpacity = 1;

const dy = 30 ;
const dx = 4 ;
const colDx = 70 ;
const colDy = 15 ;
const margeLeft = 40 ;
const margeTop = 10 ;
const yPerCol = Math.ceil((svgHeight-margeTop)/dy)-1;
const svgCanvasHeight = yPerCol * dy;

/* functions to calculatate the (x,y) location */
const locationX = (d,i) => location(d,i,'x');
const locationY = (d,i) => location(d,i,'y');
const location = (d,i,attr) => {

   const xValue = d.size;
   const yValue = i;
  
   const absX = margeLeft + xValue * dx*0 ;
   const absY = margeTop  + yValue * dy ;
   const col = absY/svgCanvasHeight;
   const relY = absY%svgCanvasHeight;
   const r = attr==='x' ? absX + col * colDx
                        : relY + (col%2?colDy:0);
   console.log(r);
   return r;     
}



let wordCloud = [];

let svg = select('#canvas')
  .append('svg')
    .attr('width',svgWidth)
    .attr('height',svgHeight);

const refresh = () => svg.selectAll('text')
      .data(wordCloud)
    .enter()
      .append("text")
		  .attr('x', flyInFromX)
      .attr('y', flyInFromY)
      .attr('font-size', flyInFromFontSize)
      .attr('fill',flyInFromColor)
      .attr('fill-opacity',flyInFromOpacity)

      .attr('text-anchor','middle')
      .attr('dominant-baseline','middle')
      .text(d =>  d.text)
      .transition()
	  	  .duration(transitionDurationMS)
        .attr('x', locationX )
        .attr('y', locationY )
        .attr('font-size', wordSize )
        .attr('fill', wordColor )
        .attr('fill-opacity',wordOpacity )
;

/* a helper function to animate at a given rate for a given duration */
const playAnimation = (animationFunc, rateMS, durationMS=null) => {
  const myAnimation = setInterval( animationFunc, rateMS );
  if (durationMS) setTimeout( () => clearInterval(myAnimation) , durationMS );
}

/* play the animation for 20 seconds */
playAnimation( animationFunction, animationRateMS, animationRateDurationMS );
